@@ -1,4 +1,4 @@
-! { dg-do run { xfail sparc*-sun-solaris2.* } }
+! { dg-do run }
 ! { dg-require-effective-target fortran_large_real }
 ! PR libfortran/24685
 program large_real_kind_form_io_2
@@ -19,13 +19,13 @@ program large_real_kind_form_io_2
   if (a /= b(1)) call abort ()
   if (c /= b(2)) call abort ()
 
-  b(:) = tiny(0.0_k)
+  b(:) = nearest(tiny(0.0_k),1.0_k)
   write (tmp, *) b
   read (tmp, *) a, c
   if (a /= b(1)) call abort ()
   if (c /= b(2)) call abort ()
 
-  b(:) = -tiny(0.0_k)
+  b(:) = nearest(-tiny(0.0_k),-1.0_k)
   write (tmp, *) b
   read (tmp, *) a, c
   if (a /= b(1)) call abort ()
