@@ -9,7 +9,7 @@ module anything
   interface
     function  lc(string )
       character(len=*), intent(in)  :: string 
-      character(len=20)    :: lc
+      character(len=len(string ))    :: lc
     end function lc
   end interface
   character(len=12) :: string
@@ -24,3 +24,5 @@ end module serial
   character*15  :: buffer  
   buffer = lc ("Have a Nice DAY")
   end
+
+! { dg-final { cleanup-modules "anything serial" } }
